const { isArray, isObject, isString, mapValues } = require('lodash')
const { getResourceString } = require('./src/lang')
const getResourceStringDE = getResourceString('de-de')

class ClientError extends Error {
  constructor (message) {
    super()
    Error.captureStackTrace(this, this.constructor)
    this.name = this.constructor.name
    this.message = message
    this.meta = {}
  }
}

class AuthorizationError extends ClientError {}

class NotFoundError extends ClientError {}

class ForbiddenError extends ClientError {
  constructor (reason) {
    super('Forbidden')
    if (reason) this.reason = reason
  }
}

class ValidationError extends ClientError {
  constructor (message, meta) {
    super(isString(message) ? message : getResourceStringDE('ERROR_INPUT_VALIDATION_FAILED'))
    if (isObject(message)) this.meta = message
    if (meta) this.meta = meta

    this.meta = mapValues(this.meta, value => {
      if (isArray(value)) {
        return value
      } else if (typeof value === 'object') {
        return [value]
      } else {
        return [{ code: value, message: getResourceStringDE('ERROR_INPUT_VALIDATION_FAILED') }]
      }
    })

    if (this.meta && typeof this.meta === 'object' && isArray(this.meta[Object.keys(this.meta)[0]])) {
      this.message = this.meta[Object.keys(this.meta)[0]][0].message || getResourceStringDE('ERROR_INPUT_VALIDATION_FAILED')
    }
  }
}

class JSONSchemaValidationError extends ClientError {
  constructor (message, ajvError) {
    super(isString(message) ? message : getResourceStringDE('ERROR_INPUT_VALIDATION_FAILED'))
    if (isObject(message)) this.meta = message
    if (ajvError) this.meta = ajvError

    const keys = Object.keys(this.meta)
    if (keys.length === 1) {
      const [key] = keys
      const transformedErrors = mapValues(this.meta, ajvErr => {
        return ajvErr.reduce((result, { params, dataPath }) => {
          const subKey = `${key}${dataPath}`
          const errors = !isArray(params) ? [params] : params
          result[subKey] = isArray(result[subKey]) ? result[subKey].concat(errors) : [].concat(errors)
          return result
        }, {})
      })
      this.meta = transformedErrors[key]
    } else {
      this.meta = mapValues(this.meta, ajvErr => {
        return ajvErr.map(error => error.params)
      })
    }
  }
}







module.exports = {
  ClientError,
  AuthorizationError,
  NotFoundError,
  ValidationError,
  JSONSchemaValidationError,
  ForbiddenError,
 
 
}
